import type { GetServerSideProps, GetStaticPaths, GetStaticProps, NextComponentType, PageConfig } from '../../../../../types';
import type { PagesRouteDefinition } from '../../route-definitions/pages-route-definition';
import { RouteModule, type RouteModuleOptions } from '../route-module';
/**
 * The userland module for a page. This is the module that is exported from the
 * page file that contains the page component, page config, and any page data
 * fetching functions.
 */
export declare type PagesUserlandModule = {
    /**
     * The exported page component.
     */
    readonly default: NextComponentType;
    /**
     * The exported page config.
     */
    readonly config?: PageConfig;
    /**
     * The exported `getStaticProps` function.
     */
    readonly getStaticProps?: GetStaticProps;
    /**
     * The exported `getStaticPaths` function.
     */
    readonly getStaticPaths?: GetStaticPaths;
    /**
     * The exported `getServerSideProps` function.
     */
    readonly getServerSideProps?: GetServerSideProps;
};
export declare type PagesRouteModuleOptions = RouteModuleOptions<PagesRouteDefinition, PagesUserlandModule>;
declare class PagesRouteModule extends RouteModule<PagesRouteDefinition, PagesUserlandModule> {
    setup(): Promise<void>;
    handle(): Promise<Response>;
}
export default PagesRouteModule;
